/*    _   _ _ _____ _    _              _____     _ _     ___ ___  _  __
 *   /_\ | | |_   _| |_ (_)_ _  __ _ __|_   _|_ _| | |__ / __|   \| |/ /
 *  / _ \| | | | | | ' \| | ' \/ _` (_-< | |/ _` | | / / \__ \ |) | ' <
 * /_/ \_\_|_| |_| |_||_|_|_||_\__, /__/ |_|\__,_|_|_\_\ |___/___/|_|\_\
 *                             |___/
 *
 * Copyright 2018 AllThingsTalk
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <WiFiNINA.h>
//#include <RTCZero.h>
#include <PubSubClient.h>
#include <ArduinoJson.h>
#include <ATT_IOT.h>
#include <Wire.h>
#include <SPI.h>  // required to have support for signed/unsigned long type. 
#include "DHT.h"
#include "keys.h"

// Define http and mqtt endpoints
#define http "api.allthingstalk.io"  // API endpoint
#define mqtt "api.allthingstalk.io"  // broker

void callback(char* topic, byte* payload, unsigned int length);

ATTDevice device(DEVICE_ID, DEVICE_TOKEN);

int status = WL_IDLE_STATUS;

#define RELAY 0
#define DHTPIN 2     // what pin we're connected to

// Uncomment whatever type you're using!
#define DHTTYPE DHT11   // DHT 11 
//#define DHTTYPE DHT22   // DHT 22  (AM2302)
//#define DHTTYPE DHT21   // DHT 21 (AM2301)

// Connect pin 1 (on the left) of the sensor to +5V
// Connect pin 2 of the sensor to whatever your DHTPIN is
// Connect pin 4 (on the right) of the sensor to GROUND
// Connect a 10K resistor from pin 2 (data) to pin 1 (power) of the sensor

DHT dht(DHTPIN, DHTTYPE);



WiFiClient client;
PubSubClient pubSub(mqtt, 1883, callback, client);

void setupWiFi(const char* ssid, const char* password)
{
  // check for the WiFi module:
  if (WiFi.status() == WL_NO_MODULE) {
    Serial.println("Communication with WiFi module failed!");
    // don't continue
    while (true);
  }

  // attempt to connect to Wifi network:
  while (status != WL_CONNECTED) {
    Serial.print("Attempting to connect to SSID: ");
    Serial.println(ssid);
    // Connect to WPA/WPA2 network. Change this line if using open or WEP network:
    status = WiFi.begin(ssid, password);

    // wait 10 seconds for connection:
    delay(2000);
  }
  Serial.println("Connected to wifi");
  printWifiStatus();
}

void printWifiStatus() {
  // print the SSID of the network you're attached to:
  Serial.print("SSID: ");
  Serial.println(WiFi.SSID());

  // print your WiFi shield's IP address:
  IPAddress ip = WiFi.localIP();
  Serial.print("IP Address: ");
  Serial.println(ip);

  // print the received signal strength:
  long rssi = WiFi.RSSI();
  Serial.print("signal strength (RSSI):");
  Serial.print(rssi);
  Serial.println(" dBm");
}

void setup() 
{
    Serial.begin(9600); 
    Serial.println("send temperature & humidity to ATTALK Maker!");

    pinMode(RELAY, OUTPUT);
    // Power grove connectors
    pinMode(10, OUTPUT);
    digitalWrite(10, HIGH);

    dht.begin();
    Serial.println("DHT initialized!");
    delay(2000);

    //Time to connect to the wifi
    setupWiFi(WIFI_SSID, WIFI_PASSWORD);
     
    // Connect to the DS IoT Cloud MQTT Broker
    Serial.println("Subscribe MQTT");
    
    while(!device.subscribe(pubSub))  // Subscribe to mqtt
    Serial.println("retrying");
}

unsigned long timer;


void loop() 
{

  unsigned long curTime = millis();
  if (curTime > (timer + 10000))    {
    
    // Reading temperature or humidity takes about 250 milliseconds!
    // Sensor readings may also be up to 2 seconds 'old' (its a very slow sensor)
    float h = dht.readHumidity();
    float t = dht.readTemperature();

    // check if returns are valid, if they are NaN (not a number) then something went wrong!
    if (isnan(t) || isnan(h)) 
    {
        Serial.println("Failed to read from DHT");
    } 
    else 
    {
        Serial.print("Humidity: "); 
        Serial.print(h);
        Serial.println(" %\t");

        device.send(String(h), "humidity");
        
        Serial.print("Temperature: "); 
        Serial.print(t);
        Serial.println(" *C");

        device.send(String(t), "temperature");
    }
    timer = curTime;
  }
  device.process();
}

/****
 * Callback function
 * Handle messages that were sent from the AllThingsTalk cloud to this device
 */
void callback(char* topic, byte* payload, unsigned int length) 
{ 
  String msgString; 
  String assetName = device.getAssetName(topic, strlen(topic));
  {
    char message_buff[length + 1];
    strncpy(message_buff, (char*)payload, length);
    message_buff[length] = '\0';
          
    msgString = String(message_buff);
    msgString.toLowerCase();

    Serial.print("Payload: ");
    Serial.println(msgString);
    Serial.print("Topic  : ");
    Serial.println(topic);
  }

  {
    if(assetName == "relay")
    {
      if (msgString.indexOf("false") > -1) {
        digitalWrite(RELAY, LOW);
        device.send("false", "relay");
      }
      else if (msgString.indexOf("true") > -1) {
        digitalWrite(RELAY, HIGH);
        device.send("true", "relay");
      }
    }
  }
}
